﻿---------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("Assessment")
local SML = AceLibrary("SharedMedia-1.0")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["History"] = true,
	["Number of fights to keep detailed information for. Use 0 to keep all."] = true,
	["Reset"] = true,
	["Click to reset fight data."] = true,
	["Data Options"] = true,
	["Data options."] = true,

    ["FilterAll"] = "Show All",
	["FilterNonZero"] = "Show Non-Zero Values",
} end)

-----------------------------------------------------------------------------

Assessment = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceModuleCore-2.0")
Assessment.Debug = (UnitName("player") == "Roartindon") and function(_, text) ChatFrame3:AddMessage(text) end or function() end
Assessment:RegisterDB("AssessmentDB", "AssessmentDBPerCharacter")
Assessment:RegisterDefaults('char', { fightData = { } } )
Assessment:RegisterDefaults('profile', { 
	history = 10,
} )

-----------------------------------------------------------------------------

Assessment.Options = 
{
	handler = Assessment,
	type = 'group',
	args = 
	{
		reset =
		{
			type = "execute",
			name = L["Reset"],
			desc = L["Click to reset fight data."],
			func = function() Assessment:ResetFightData() end,
			order = 1,
		},
		history =
		{
			type = "range",
			name = L["History"],
			desc = L["Number of fights to keep detailed information for. Use 0 to keep all."],
			max = 20,
			min = 0,
			step = 1,
			get = function() return Assessment.db.profile.history end,
			set = function(value) Assessment.db.profile.history = value end,
			order = 10,
		},
		dataSpacer = 
		{
			type = 'header',
			order = 90,
		},
		data =
		{
			type = 'group',
			name = L["Data Options"],
			desc = L["Data options."],
			disabled = function() return not next(Assessment.Options.args.data.args) end,
			args = { },
			order = 100,
		}
	},
}

Assessment:RegisterChatCommand({"/as", "/assessment"}, Assessment.Options)

-----------------------------------------------------------------------------

local fightData
Assessment.ListType = { }
Assessment.FilterType = { }
Assessment.DefaultDisplaySets = { }
Assessment.ColorType = { }
Assessment.DefaultTags = { }

-----------------------------------------------------------------------------

Assessment.FilterType.All =
{
	tag			= "1",
	description	= L["FilterAll"],
}

Assessment.FilterType.NonZero =
{
	tag			= "barValue ~= 0",
	description	= L["FilterNonZero"],
}

-----------------------------------------------------------------------------

function Assessment:OnInitialize()
	fightData = self.db.char.fightData
	
	if not fightData[0] then
		fightData[0] = self:NewFightDataTable()
	end
end

function Assessment:OnEnable()
end

function Assessment:OnDisable()
end

-----------------------------------------------------------------------------

function Assessment:ResetFightData()
	for i=0,#fightData do
		fightData[i] = self:ReleaseTable(fightData[i], true)
	end
	fightData[0] = self:NewFightDataTable()
	self:TriggerEvent("Assessment:ResetFightData")
end

function Assessment:NewFightDataTable()
	local data = self:NewTable()
	data.dataUpdateTimes = self:NewTable()
	self:TriggerEvent("Assessment:NewFightDataTable", data)
	return data
end

-----------------------------------------------------------------------------

function Assessment:StartNewFight()
	table.insert(fightData, 1, self:NewFightDataTable())
	
	if self.db.profile.history ~= 0 then
		for i=#fightData, self.db.profile.history+1, -1 do
			fightData[i] = self:ReleaseTable(fightData[i], true)
		end
	end
end

-----------------------------------------------------------------------------

local dataTablePool = { }

function Assessment:NewTable()
	local recycledTable = next(dataTablePool)
	if recycledTable then
		dataTablePool[recycledTable] = nil
		return recycledTable
	else
		return { }
	end
end

function Assessment:ReleaseTable(t, recursive)
	for k,v in pairs(t) do
		if recursive and type(v) == "table" then
			self:ReleaseTable(v, true)
		end
		t[k] = nil
	end
	dataTablePool[t] = true
end

-----------------------------------------------------------------------------

function Assessment:GetFightData(index)
	if not fightData then return nil end
	return fightData[index]
end

-----------------------------------------------------------------------------

local IterateUpdateDataTables
do
	local index

	local function DataTableIterator()
		if index < 1 then
			index = index+1
			return fightData[index]
		end
	end

	function IterateUpdateDataTables()
		index = -1
		return DataTableIterator
	end
end

-----------------------------------------------------------------------------

Assessment.modulePrototype.IterateUpdateDataTables = IterateUpdateDataTables
Assessment.modulePrototype.NewTable = Assessment.NewTable
Assessment.modulePrototype.ReleaseTable = Assessment.ReleaseTable

function Assessment.modulePrototype:Debug(message)
	Assessment:Debug(message)
end

function Assessment.modulePrototype:CancelEvent(eventName)
	if self:IsEventRegistered(eventName) then
		self:UnregisterEvent(eventName)
	end
end

-----------------------------------------------------------------------------
